# -----------------------------------------------------------------------------
# Center a specified view using the current mouse position.
# Center a specified view on a selected peak.
#
import Tkinter

import pyutil
import sparky
import sputil
import tkutil

# -----------------------------------------------------------------------------
#
class view_center_dialog(tkutil.Dialog):

  def __init__(self):

    self.view_to_center = None
    self.select_callback = None
    
    tkutil.Dialog.__init__(self, 'View Center')

    explain = ('Specify a view window to be centered\n' +
               'by the cm or cp commands.  The cm command\n' +
               'centers the view at the point under the mouse\n'
               'when the command is invoked in any window.\n' +
               'The cp command centers the view on the selected\n' +
               'peak.')
    w = Tkinter.Label(self.top, text = explain, justify = 'left')
    w.pack(side = 'top', anchor = 'w')

    sparky.add_command('cm', '',
                       'import centerview; centerview.center_view_on_mouse()')
    sparky.add_command('cp', '',
                       'import centerview; centerview.center_view_on_peak()')

    self.center_view = sputil.view_menu(self.top, 'View to center: ')
    self.center_view.frame.pack(side = 'top', anchor = 'w')

    ps = tkutil.checkbutton(self.top, 'Center when peak selected?', 0)
    ps.button.pack(side = 'top', anchor = 'w')
    self.peak_select = ps

    br = tkutil.button_row(self.top,
			   ('Ok', self.ok_cb),
                           ('Apply', self.apply_cb),
                           ('Close', self.close_cb),
                           ('Help', sputil.help_cb('CenterViews')),
			   )
    br.frame.pack(side = 'top', anchor = 'w')
    
  # ---------------------------------------------------------------------------
  #
  def ok_cb(self):

    self.apply_cb()
    self.close_cb()
    
  # ---------------------------------------------------------------------------
  #
  def apply_cb(self):

    self.view_to_center = sputil.name_to_view(self.center_view.get())

    if self.peak_select.variable.get():
      if self.select_callback == None:
        self.select_callback = self.peak_selected_cb
        sparky.add_selection_callback(self.select_callback)
    elif self.select_callback:
      sparky.remove_selection_callback(self.select_callback)
      self.select_callback == None
    
  # ---------------------------------------------------------------------------
  #
  def peak_selected_cb(self):

    if self.is_window_destroyed():
      if self.select_callback:
        sparky.remove_selection_callback(self.select_callback)
      return
    self.center_on_peak()
      
  # ---------------------------------------------------------------------------
  #
  def center_on_mouse(self):

    pointer_view = sparky.selected_view()
    if pointer_view:
      p = pointer_view.pointer_position
      if p:
        self.center_on_point(pointer_view.spectrum, p)
    
  # ---------------------------------------------------------------------------
  # Center the view specified by the option menu on the selected peak.
  #
  def center_on_peak(self):

    peak = sputil.selected_peak()
    if peak:
      self.center_on_point(peak.spectrum, peak.frequency)
    
  # ---------------------------------------------------------------------------
  #
  def center_on_point(self, spectrum, point):

    view_to_center = self.view_to_center
    if view_to_center:
      center = self.map_point(spectrum, point,
                              view_to_center.spectrum, view_to_center.center)
      view_to_center.center = center
    
  # ---------------------------------------------------------------------------
  # Map a point from one spectrum to another matching nucleus types.
  # A default destination point is specified and components are only
  # changed if the nucleus type is unique in the source spectrum or
  # the same axis number of the source spectrum has the correct nucleus.
  #
  def map_point(self, from_spectrum, from_p, to_spectrum, to_p_default):

    from_nuclei = from_spectrum.nuclei
    to_nuclei = to_spectrum.nuclei
    to_p = list(to_p_default)
    for to_a in range(to_spectrum.dimension):
      from_a = pyutil.unique_element_index(to_nuclei[to_a], from_nuclei)
      if (from_a == None and
          to_a < from_spectrum.dimension and
          from_nuclei[to_a] == to_nuclei[to_a]):
        from_a = to_a
      if from_a != None:
        to_p[to_a] = from_p[from_a]
    return to_p
    
# -----------------------------------------------------------------------------
#
def center_view_on_mouse():
  tkutil.the_dialog(view_center_dialog).center_on_mouse()
def center_view_on_peak():
  tkutil.the_dialog(view_center_dialog).center_on_peak()
def center_view_setup():
  tkutil.the_dialog(view_center_dialog).show_window(1)

