# -----------------------------------------------------------------------------
# Read Mardigras constraint file lower and upper distance bounds.
#
import os
import string

import atoms
import noesy
import tkutil

# -----------------------------------------------------------------------------
#
mardi_table = {}
def bounds(mardi_path, stoppable = tkutil.Not_Stoppable()):
  
  if not mardi_table.has_key(mardi_path):
    mardi_table[mardi_path] = mardigras_bounds(mardi_path, stoppable)
  return mardi_table[mardi_path]

# -----------------------------------------------------------------------------
#
class mardigras_bounds:

  def __init__(self, mardi_path, stoppable = tkutil.Not_Stoppable()):

    self.mardi_path = mardi_path
    self.bounds_table = read_bounds(mardi_path, stoppable)

    gatable = {}
    for num_atom_1, num_atom_2 in self.bounds_table.keys():
      gatable[(None, num_atom_1[1])] = 1
      gatable[(None, num_atom_2[1])] = 1
    self.atom_translations = \
      atoms.guess_atom_name_translations(mardi_path, gatable.keys())

  # ---------------------------------------------------------------------------
  #
  def distance_bounds(self, peak):

    if peak.is_assigned:
      res = noesy.proton_resonances(peak)
      translate = self.atom_translations
      return self.bounds(translate.resonance_number_atom(res[0]),
                         translate.resonance_number_atom(res[1]))
    return None

  # ---------------------------------------------------------------------------
  #
  def bounds(self, num_atom_1, num_atom_2):

    try:
      return self.bounds_table[(num_atom_1, num_atom_2)]
    except KeyError:
      return None

# -----------------------------------------------------------------------------
#
def read_bounds(mardi_path, stoppable):

  mardi_file = open(mardi_path, 'r')
  lines = mardi_file.readlines()
  mardi_file.close()

  bounds = {}
  stoppable.stoppable_loop('reading ' + os.path.basename(mardi_path), 100)
  for line in lines:
    stoppable.check_for_stop()
    line_fields = parse_mardi_line(line)
    if line_fields:
      n1, atom1, n2, atom2, lower, upper = line_fields
      na1 = (n1, atom1)
      na2 = (n2, atom2)
      b = (lower, upper)
      bounds[(na1, na2)] = b
      bounds[(na2, na1)] = b

  return bounds

# -----------------------------------------------------------------------------
# Return n1, atom1, n2, atom2, lower, upper
#
def parse_mardi_line(line):

  if line[:4] == 'ATOM':	# header line
    return None

  atom1 = string.strip(line[0:4])
  n1 = string.atoi(line[4:7])
  atom2 = string.strip(line[8:12])
  n2 = string.atoi(line[12:15])
  lower = string.atof(line[15:23])
  upper = string.atof(line[23:31])

  return n1, atom1, n2, atom2, lower, upper
