# -----------------------------------------------------------------------------
# Move one assigned peak and have all other peaks along its resonance lines
# moved by the same amount.  This is used to assign a new spectrum taken under
# new experimental conditions when a previous spectrum has already been
# assigned.  You can select all peaks from the old spectrum (pa) and copy and
# paste them to the new spectrum (accelerators oc and op).  Then start this
# tool and move a peak for each resonance line to its shifted location.
#

import Tkinter

import sparky
import sputil
import tkutil

# -----------------------------------------------------------------------------
# Doesn't work for peak groups.
# Should be redesigned to use only one spectrum.
#
def move_peaks(old_spectrum, new_spectrum, new_resonance, shift):

  count = 0
  for np in new_resonance.peak_list():
    if np.spectrum == new_spectrum and not np.peaklets():
      op = old_spectrum.find_peak(np.assignment)
      if op:
	ofreq = op.frequency
	nfreq = list(np.frequency)
	res = np.resonances()
	for a in range(new_spectrum.dimension):
	  if res[a] == new_resonance:
	    nfreq[a] = ofreq[a] + shift
	np.frequency = nfreq
	count = count + 1
  return count

# -----------------------------------------------------------------------------
#
class peak_mover(tkutil.Dialog):

  # ---------------------------------------------------------------------------
  #
  def __init__(self):

    tkutil.Dialog.__init__(self, 'Peak Resonance Mover')

    self.old_spectrum = sputil.spectrum_menu(self.top, 'Old Spectrum: ')
    self.old_spectrum.frame.pack(side = 'top', anchor = 'w')

    self.new_spectrum = sputil.spectrum_menu(self.top, 'New Spectrum: ')
    self.new_spectrum.frame.pack(side = 'top', anchor = 'w')

    cb = tkutil.checkbutton(self.top, 'On / Off', 0)
    self.onoff = cb.variable
    cb.button.pack(side = 'top', anchor = 'w')
    cb.button['command'] = self.onoff_cb
    self.callback = self.move_cb

    self.message = Tkinter.Label(self.top)
    self.message.pack(side = 'top', anchor = 'w')

    br = tkutil.button_row(self.top,
			   ('Close', self.close_cb),
                           ('Help', sputil.help_cb('ShiftRes')),
			   )
    br.frame.pack(side = 'top', anchor = 'w')

  # ---------------------------------------------------------------------------
  #
  def onoff_cb(self):

    if self.onoff.get():
      sparky.add_peak_moved_callback(self.callback)
    else:
      sparky.remove_peak_moved_callback(self.callback)

  # ---------------------------------------------------------------------------
  #
  def move_cb(self, peak):

    if self.is_window_destroyed():
      sparky.remove_peak_moved_callback(self.callback)
      return

    old_spectrum = sputil.name_to_spectrum(self.old_spectrum.get())
    new_spectrum = sputil.name_to_spectrum(self.new_spectrum.get())
    if peak.spectrum == new_spectrum:
      opeak = old_spectrum.find_peak(peak.assignment)
      if opeak:
	sparky.remove_peak_moved_callback(self.callback)
	total = 0
	res = peak.resonances()
	dim = new_spectrum.dimension
	for a in range(dim):
	  shift = peak.frequency[a] - opeak.frequency[a]
	  count = 1
	  count = move_peaks(old_spectrum, new_spectrum, res[a], shift)
	  total = total + count
	sparky.add_peak_moved_callback(self.callback)
	self.message['text'] = 'Moved %d additional peaks' % (total - dim)

  # ---------------------------------------------------------------------------
  #
  def close_cb(self):

    self.onoff.set(0)
    tkutil.Dialog.close_cb(self)

# -----------------------------------------------------------------------------
#
def show_move_peak_dialog():
  tkutil.the_dialog(peak_mover).show_window(1)
